'use strict';

var fs = require('fs');
var spawn = require('child_process').spawn;

var ExpectedError = require('../modules/errors.js').ExpectedError;

// TODO: move more logics from ../commands/system.js to this file (or its
// sub-files), and keep that one as simple as possible.

function getUptime() {
    var fd = uv.fs_open('/proc/uptime', 'rn', parseInt('666', 8));

    var text = uv
        .fs_read(fd, 1024, 0)
        .toString()
        .trim();

    uv.fs_close(fd);

    return Number((text.match(/[\d.]+/) || [])[0]) || undefined;
}

exports.getUptime = getUptime;

function setWiFi(ssid, password, callback) {
    try {
        spawn('/usr/sbin/enable_wifi_sta.sh', [ssid, password], {
            cwd: '/',
            stdio: ['ignore', 'ignore', 'ignore']
        });
        callback();
    } catch (error) {
        callback(new ExpectedError('Failed to configure Wifi settings', 2, error));
    }
}

exports.setWiFi = setWiFi;

function getUsedHotspots(withCredentials) {
    var hotspots = [];

    var text;

    try {
        text = fs.readFileSync('/etc/netmanager/aps_db.conf', 'utf-8');
    } catch (error) {
        return hotspots;
    }

    var captures;
    var regex = /^#\*+#\nssid=(.+)\npasswd=(.*)\n/mg;

    while (captures = regex.exec(text)) {
        hotspots.push({
            ssid: captures[1],
            password: withCredentials ? captures[2] : undefined
        });
    }

    return hotspots;
}

exports.getUsedHotspots = getUsedHotspots;

function setUsedHotspot(ssid, callback) {
    var hotspots = getUsedHotspots(true);

    var password;
    var i;

    for (i = 0; i < hotspots.length; i++) {
        var hotspot = hotspots[i];

        if (hotspot.ssid === ssid) {
            password = hotspot.password;
            break;
        }
    }

    if (i === hotspots.length) {
        callback(new ExpectedError('No used hotspot found with SSID "' + ssid + '"'));
        return;
    }

    setWiFi(ssid, password, callback);
}

exports.setUsedHotspot = setUsedHotspot;
